{$R+,Q+}
uses testlib, SysUtils, Math;

const
	SIZE = 8;
	NO_SOLUTION = 'NO SOLUTION';
	
var
	board : array[1..SIZE,1..SIZE] of char;
	s : string;
	juryHasSol, partHasSol : boolean;
	i, j : integer;
	numPieces : integer;
	
	row1, col1, row2, col2 : integer;

procedure ParseLocation(s : string; var row, col : integer);
begin
	if (length(s) <> 2) or not (s[1] in ['a'..'h']) or not (s[2] in ['1'..'8']) then
		Quit(_PE, 'Invalid location format: ' + s);
	row := 9 - (ord(s[2]) - ord('0'));
	col := ord(s[1]) - ord('a') + 1;
end;

function CheckLine(row1, col1, row2, col2 : integer) : boolean;
var	
	drow, dcol : integer;
	row, col : integer;
	
begin
	if row1 > row2 then
		drow := -1
	else if row1 < row2 then
		drow := +1
	else
		drow := 0;	
		
	if col1 > col2 then
		dcol := -1
	else if col1 < col2 then
		dcol := +1
	else
		dcol := 0;	
		
	row := row1;
	col := col1;
	result := false;
		
	repeat
		inc(row, drow);
		inc(col, dcol);
		if (row = row2) and (col = col2) then
			break;
		if board[row, col] <> '.' then
			exit;
	until false;
	
	result := true;	
end;

function CheckRookMove(row1, col1, row2, col2 : integer) : boolean;
begin
	if (row1 <> row2) and (col1 <> col2) then begin
		result := false;
		exit;
	end;
	result := CheckLine(row1, col1, row2, col2);	
end;
	
function CheckBishopMove(row1, col1, row2, col2 : integer) : boolean;
var
	alpha1, beta1 : integer;
	alpha2, beta2 : integer;
	
begin
	alpha1 := row1 + col1;
	beta1 := row1 - col1;
	
	alpha2 := row2 + col2;
	beta2 := row2 - col2;
	
	if (alpha1 <> alpha2) and (beta1 <> beta2) then begin
		result := false;
		exit;
	end;
	result := CheckLine(row1, col1, row2, col2);	
end;
	
function CheckQueenMove(row1, col1, row2, col2 : integer) : boolean;
begin
	result := CheckRookMove(row1, col1, row2, col2) or CheckBishopMove(row1, col1, row2, col2);
end;
	
function CheckKnightMove(row1, col1, row2, col2 : integer) : boolean;
var
	dx, dy : integer;
	
begin
	dx := min(abs(row1 - row2), abs(col1 - col2));
	dy := max(abs(row1 - row2), abs(col1 - col2));
	result := (dx = 1) and (dy = 2);
end;
	
begin
	numPieces := 0;
	for i := 1 to SIZE do begin
		s := inf.ReadString;
		for j := 1 to SIZE do begin
			board[i,j] := s[j];
			if s[j] <> '.' then
				inc(numPieces);
		end;		
	end;		
	
	juryHasSol := ans.ReadString <> NO_SOLUTION;
	
	i := 0;
	while not ouf.SeekEOF do begin
		inc(i);
		s := ouf.ReadString;
		if i = 1 then begin
			partHasSol := s <> NO_SOLUTION;
			if not juryHasSol and not partHasSol then
				Quit(_OK, 'No solution');
			if juryHasSol and not partHasSol then
				Quit(_WA, 'Solution exists but was not found');
		end;
		
		j := pos(':', s);
		if j = 0 then
			Quit(_PE, 'Invalid move format: ' + s);
			
		ParseLocation(copy(s, 1, j - 1), row1, col1);
		ParseLocation(copy(s, j + 1, length(s) - j), row2, col2);
		
		if (row1 = row2) and (col1 = col2) then
			Quit(_WA, 'Wrong move: ' + s);

		if board[row1,col1] = '.' then
			Quit(_WA, 'Attempt to move a nonexistent piece: ' + s);

		if board[row2,col2] = '.' then
			Quit(_WA, 'Attempt to attack a nonexistent piece: ' + s);

		case board[row1,col1] of
			'R' : if not CheckRookMove(row1, col1, row2, col2) then
				Quit(_WA, 'Invalid rook move: ' + s);
			'Q' : if not CheckQueenMove(row1, col1, row2, col2) then
				Quit(_WA, 'Invalid queen move: ' + s);
			'B' : if not CheckBishopMove(row1, col1, row2, col2) then
				Quit(_WA, 'Invalid bishop move: ' + s);
			'K' : if not CheckKnightMove(row1, col1, row2, col2) then
				Quit(_WA, 'Invalid knight move: ' + s);
		end;
		
		board[row2,col2] := board[row1,col1];
		board[row1,col1] := '.';
	end;

	if i <> numPieces - 1 then	
		Quit(_WA, format('Too few moves: %d instead of %d', [i, numPieces - 1]));

	if not juryHasSol then	
		Quit(_Fail, 'Solution exists');
		
	Quit(_OK, format('%d piece(s)', [numPieces]));	
end.
